#! /usr/bin/perl

# Stuart Levy, salevy@illinois.edu, 2011, for Laird Whitehill

if(@ARGV == 0 && -t STDIN) {
    print STDERR
"Usage:	perl wiki2elements.pl  transneptunian.wiki > transneptunian.elements
	perl elements2speck.pl transneptunian.elements > transneptunian.speck
Reads a file copied-and-pasted from Wikipedia pages of transneptunian objects as linked from 
  http://en.wikipedia.org/wiki/List_of_trans-Neptunian_objects
and writes a comma-separated file in the form read by elements2speck.pl.  See it for details.
Each paragraph (separated by blank lines) defines one object.
Pays attention to lines that look like the following (and ignores other lines, so clutter is fine):
  Named after 
  MPC designation
  Semi-major axis   ... AU   (looks for a number preceding \"AU\", since entries may include multiple units)
  Eccentricity
  Inclination
  Longitude of ascending node
  Argument of perihelion
(and optionally, for Laird Whitehill's purposes)
  Label longitude
  Orbit color
  Orbit points
See \"elements2speck.pl\" for details on label longitude, orbit color, orbit points.
";
    exit(1);
}

sub dump {
    $name = $mpc if($name eq "" && $mpc ne "");
    if(defined($name) && defined($a) && defined($ecc) && defined($incl) && defined($node) && defined($peri)) {
	$name =~ s/^\s*\(?\d+\)?\s*//;
	$cindex = "-" unless defined($cindex);
	$labellon = "-" unless defined($labellon);
	$npoints = "-" unless defined($npoints);
	print "$name,$a,$ecc,$incl,$node,$peri,$cindex,$labellon,$npoints # $mpc\n";
    } elsif($name ne "" || defined($a) || defined($ecc) || defined($incl) || defined($node) || defined($peri)) {
	print "# Partial record: name $name a $a ecc $ecc incl $incl node $node peri $peri\n";
    }
    $name = $mpc = $a = $ecc = $incl = $node = $peri = $cindex = $labellon = $npoints = undef;
}
	
    

while(<>) {
    s/#.*//;
    &dump() if(/^\s+$/);
    # Change Unicode em-dash to ASCII hyphen so things like  −1.2799 (semimajor axis of 'Oumuamua) are read correctly
    s/−/-/g;
    s/#.*//;
    if(/Semi-?major.*[\s\(](-?\d+[.\d]*) AU/i) {
	$a = $1;
    }
    if(/MPC designation\s+(.*)/i) {
	$mpc = $1;
    }
    if(/Named (after|for)\s+(.*)/i) {
	$name = $2;
    }
    if(/Eccentricity\s+(\S+)/i) {
	$ecc = $1;
    }
    if(/Inclination\s+(\d+[.\d]*)/i) {
	$incl = $1;
    }
    if(/Longitude of ascending node\s+(\d+[.\d]*)/i) {
	$node = $1;
    }
    if(/Argument of perihelion\s+(\d+[.\d]*)/i) {
	$peri = $1;
    }
    if(/(Orbit ?color|Color ?index)\s+(\d+)/i) {
	$cindex = $1;
    }
    if(/Label longitude\s+(\S.*)/i) {	# <degrees> or p+<degrees> or <N>c+<degrees>
	$labellon = $1;
    }
    if(/Orbit ?points\s+(\d+)/i) {
	$npoints = $1;
    }
}

&dump();
